package de.tum.wzw.ttss;

import de.tum.wzw.dataprocessing.ClassifierModul;
import de.tum.wzw.iohandling.ARFFReader;
import de.tum.wzw.iohandling.FastaEntry;
import de.tum.wzw.iohandling.FastaFile;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Enumeration;
import weka.classifiers.Classifier;
import weka.core.Attribute;
import weka.core.Instances;

public class FinalClassifier
  extends ClassifierModul
{
  public static int start = 0;
  public static int length = 25;
  private String arff = "";
  private String path = null;
  private String url = null;
  private Instances training = null;
  private Classifier cModel = null;
  ArrayList<String> attstouse = null;
  
  public void setClassifer(String classifier, String path, String arff, String url)
    throws Exception
  {
    this.arff = arff;
    this.path = path;
    this.url = url;
    this.cModel = GetClassifier.getClassifier(classifier);
    log("classifier loaded:\t" + classifier);
  }
  
  private static boolean logging = false;
  private static String classifier = "de.tum.wzw.ttss.NaiveBayesCut";
  private String restriction;
  private double cutoff;
  
  private static void log(String message)
  {
    if (logging) {
      System.err.println(message);
    }
  }
  
  private String version = "combined";
  
  public void initialize(String restrict, String path, String url)
    throws Exception
  {
    if (restrict.equalsIgnoreCase("selective"))
    {
      this.cutoff = 0.9999D;
      this.restriction = (restrict + " ");
    }
    else if (restrict.equalsIgnoreCase("sensitive"))
    {
      this.cutoff = 0.95D;
      this.restriction = (restrict + " ");
    }
    else
    {
      this.cutoff = Double.parseDouble(restrict.substring(8));
      this.restriction = "";
    }
    setClassifer(classifier, path, this.version.toUpperCase() + ".arff", url);
  }
  
  public String type()
  {
    String type = "Type III Effector prediction " + this.restriction + "with ";
    if (this.version.equalsIgnoreCase("std")) {
      type = type + "standard";
    } else {
      type = type + this.version;
    }
    type = type + " set ";
    return type;
  }
  
  public String description()
  {
    String desc = "<html><br>This module is the classificator to predict possible effector proteines of the type three secretion system. The Prediction schema is trained with all effector sequences derived from ";
    if (this.version.equalsIgnoreCase("std")) {
      desc = desc + "E. coli, Salmonella, Chlamydia, Yersinia and Pseudomonas. (Animal pathogens & Plant symbiont)";
    } else if (this.version.equalsIgnoreCase("plant")) {
      desc = desc + "the plant symbiont Pseudomonas syringae.";
    } else {
      desc = desc + "E. coli, Salmonella, Chlamydia and Yersinia. (Animal/Human pathogens)";
    }
    return desc;
  }
  
  public int[] getCutterWindows()
  {
    int[] windows = { 0, 0, 24 };
    return windows;
  }
  
  public double getRestrictionValue()
  {
    return this.cutoff;
  }
  
  private void initClassifier()
    throws Exception
  {
    if (this.url == null) {
      this.training = new ARFFReader().readfromFile(this.path, this.arff, this.version);
    } else {
      this.training = new ARFFReader().readfromURL(this.url, this.arff);
    }
    log("training-data loaded:\t" + this.arff);
    this.attstouse = new ArrayList();
    Enumeration<Attribute> atts = this.training.enumerateAttributes();
    int x = this.training.numAttributes() - 1;
    int i = 0;
    while (atts.hasMoreElements())
    {
      Attribute att = (Attribute)atts.nextElement();
      this.attstouse.add(att.name());
      i++;
      if (i == x) {
        break;
      }
    }
    log("used attributes\t" + this.attstouse);
    this.training.setClass((Attribute)atts.nextElement());
    this.cModel.buildClassifier(this.training);
  }
  
  public double makePrediction(String sequence)
    throws Exception
  {
    initClassifier();
    ArrayList<FastaEntry> current = new ArrayList();
    if (sequence.substring(start).length() < length) {
      throw new Exception("sequence to short! " + sequence);
    }
    FastaEntry dummy = new FastaEntry("test", "", sequence.toUpperCase());
    current.add(dummy);
    NewCutter pred = new NewCutter(current, start, length, false, Double.valueOf(0.15D));
    ReducedFeatureCreator hase = new ReducedFeatureCreator(this.attstouse);
    
    Cutter cur = hase.addFeatures(pred);
    Instances labeled = ToWeka.getUnlabeledInstances(cur, this.attstouse);
    if (!equalInstances(this.training, labeled)) {
      throw new Exception("error different schemata!");
    }
    labeled.setClassIndex(this.training.numAttributes() - 1);
    double clsLabel = this.cModel.classifyInstance(labeled.instance(0));
    return clsLabel;
  }
  
  public boolean isSecreted(double score)
  {
    if (score >= this.cutoff) {
      return true;
    }
    return false;
  }
  
  protected boolean equalInstances(Instances a, Instances b)
  {
    Enumeration aa = a.enumerateAttributes();
    Enumeration bb = b.enumerateAttributes();
    while (aa.hasMoreElements())
    {
      Attribute t1 = (Attribute)aa.nextElement();
      Attribute t2 = (Attribute)bb.nextElement();
      if (!t1.equals(t2)) {
        return false;
      }
    }
    return true;
  }
  
  public static void main(String[] args)
  {
    try
    {
      FinalClassifier myclass = new FinalClassifier();
      
      myclass.setClassifer(classifier, "main/de/tum/wzw/ttss/data/", "first25.arff.20081007", null);
      
      start = 1;
      length = 25;
      log("start: " + start);
      log("length: " + length);
      
      FastaFile testfile = new FastaFile("/home/user/nussbaumer/Dropbox/workspace/JAVA/Recompile/src/de/tum/wzw/ttss/test.fa");
      while (testfile.hasNext()) {
        FastaEntry localFastaEntry = testfile.next();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
}

